from ...typedefs cimport class_t, hash_t

# These are passed as callbacks to thinc.search.Beam
cdef int transition_state(void* _dest, void* _src, class_t clas, void* _moves) except -1

cdef int check_final_state(void* _state, void* extra_args) except -1
