"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.sk.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "Ardevop, s.r.o. je malá startup firma na území SR.",
    "Samojazdiace autá presúvajú poistnú zodpovednosť na výrobcov automobilov.",
    "Košice sú na východe.",
    "Bratislava je hlavné mesto Slovenskej republiky.",
    "Kde si?",
    "Kto je prezidentom Francúzska?",
    "Aké je hlavné mesto Slovenska?",
    "Kedy sa narodil Andrej Kiska?",
    "Včera som dostal 100€ na ruku.",
    "Dnes je nedeľa 26.1.2020.",
    "Narodil sa 15.4.1998 v Ružomberku.",
    "Niekto mi povedal, že 500 eur je veľa peňazí.",
    "Podaj mi ruku!",
]
