"""
Example sentences to test spaCy and its language models.
>>> from spacy.lang.tn.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "Apple e nyaka go reka JSE ka tlhwatlhwa ta R1 billion",
    "Johannesburg ke toropo e kgolo mo Afrika Borwa.",
    "O ko kae?",
    "ke mang presidente ya Afrika Borwa?",
    "ke eng toropo kgolo ya Afrika Borwa?",
    "Nelson Mandela o belegwe leng?",
]
