"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.ja.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "アップルがイギリスの新興企業を１０億ドルで購入を検討",
    "自動運転車の損害賠償責任、自動車メーカーに一定の負担を求める",
    "歩道を走る自動配達ロボ、サンフランシスコ市が走行禁止を検討",
    "ロンドンはイギリスの大都市です。",
]
