STOP_WORDS = set(
    """
සහ
සමග
සමඟ
අහා
ආහ්
ආ
ඕහෝ
අනේ
අඳෝ
අපොයි
අපෝ
අයියෝ
ආයි
ඌයි
චී
චිහ්
චික්
හෝ‍
දෝ
දෝහෝ
මෙන්
සේ
වැනි
බඳු
වන්
අයුරු
අයුරින්
ලෙස
වැඩි
ශ්‍රී
හා
ය
නිසා
නිසාවෙන්
බවට
බව
බවෙන්
නම්
වැඩි
සිට
දී
මහා
මහ
පමණ
පමණින්
පමන
වන
විට
විටින්
මේ
මෙලෙස
මෙයින්
ඇති
ලෙස
සිදු
වශයෙන්
යන
සඳහා
මගින්
හෝ‍
ඉතා
ඒ
එම
ද
අතර
විසින්
සමග
පිළිබඳව
පිළිබඳ
තුළ
බව
වැනි
මහ
මෙම
මෙහි
මේ
වෙත
වෙතින්
වෙතට
වෙනුවෙන්
වෙනුවට
වෙන
ගැන
නෑ
අනුව
නව
පිළිබඳ
විශේෂ
දැනට
එහෙන්
මෙහෙන්
එහේ
මෙහේ
ම
තවත්
තව 
සහ
දක්වා
ට
ගේ
එ
ක
ක්
බවත්
බවද
මත
ඇතුලු
ඇතුළු
මෙසේ
වඩා
වඩාත්ම
නිති
නිතිත්
නිතොර
නිතර
ඉක්බිති
දැන්
යලි
පුන
ඉතින්
සිට
සිටන්
පටන්
තෙක්
දක්වා
සා
තාක්
තුවක්
පවා
ද
හෝ‍
වත්
විනා
හැර
මිස
මුත්
කිම
කිම්
ඇයි
මන්ද
හෙවත්
නොහොත්
පතා
පාසා
ගානෙ
තව
ඉතා
බොහෝ
වහා
සෙද
සැනින්
හනික
එම්බා
එම්බල
බොල
නම්
වනාහි
කලී
ඉඳුරා
අන්න
ඔන්න
මෙන්න
උදෙසා
පිණිස
සඳහා
අරබයා
නිසා
එනිසා
එබැවින්
බැවින්
හෙයින්
සේක්
සේක
ගැන
අනුව
පරිදි
විට
තෙක්
මෙතෙක්
මේතාක්
තුරු
තුරා
තුරාවට
තුලින්
නමුත්
එනමුත්
වස්
මෙන්
ලෙස
පරිදි
එහෙත්
""".split()
)
